@@ -10,7 +10,16 @@ namespace Evolution {
 		Private,
 		Confidental,
 		}
-	
+
+#if USE_EDS_2_24
+	public enum CalTransparency : int {
+		None,
+		Transparent,
+		Opaque,
+		Unknown,
+	}
+#endif	
+
 	public enum CalPriority : int {
 		Undefined   = 0,
 		High        = 3,
@@ -47,6 +56,20 @@ namespace Evolution {
 		FreeBusy,
 		TimeZone
 	}
+
+#if USE_EDS_2_24
+	public enum CalComponentRangeType {
+		Single,
+		ThisPrior,
+		ThisFuture		
+	}
+
+	public struct CalComponentRange {
+		public CalComponentRangeType rangeType;
+		public CalComponentDateTime dt;
+	}
+
+#endif	
    
 	public struct CalComponentText {
 		public string value;
@@ -129,6 +152,8 @@ namespace Evolution {
 			static extern IntPtr e_cal_component_new ();
 		[DllImport("ecal")]
 			static extern void e_cal_component_set_new_vtype (IntPtr handle, CalComponentVType type);
+		[DllImport("ecal")]
+			static extern IntPtr e_cal_component_new_from_string (string calobj);
 
 		public CalComponent ()
 		{
@@ -136,6 +161,13 @@ namespace Evolution {
 			handle = e_cal_component_new ();
 			ecal = null;
 		}
+		
+		public CalComponent (string calobj)
+		{
+			new_component = true;
+			handle = e_cal_component_new_from_string (calobj);
+			ecal = null;
+		}
 
 		public CalComponent (Cal _ecal)
 		{
@@ -247,6 +279,50 @@ namespace Evolution {
 			}
 		}
 
+#if USE_EDS_2_24
+		[DllImport("ecal")]
+			static extern void e_cal_component_get_recurid (IntPtr handle, out IntPtr recurid);
+		[DllImport("ecal")]
+			static extern void e_cal_component_set_recurid (IntPtr handle, IntPtr recurid);
+
+		public DateTime Rid {
+			get 
+			{
+				IntPtr native_r = IntPtr.Zero;
+				e_cal_component_get_recurid (Handle, out native_r);
+				CalComponentRange range = (CalComponentRange) Marshal.PtrToStructure (native_r, 
+						typeof (CalComponentRange));
+
+				CalComponentDateTime cdt = range.dt;
+				Icaltimetype icalt = cdt.IcalTime;
+				DateTime dt = icalt.AsDateTimeUtc;
+
+				Marshal.FreeHGlobal (native_r);
+				return dt;
+			}
+
+			set
+			{
+				Icaltimetype icalt = new Icaltimetype ();
+				icalt.SetDateTime = value;
+
+				CalComponentDateTime cdt = new CalComponentDateTime ();
+				cdt.IcalTime = icalt;
+				cdt.Tzid = "UTC";
+				
+				CalComponentRange range = new CalComponentRange ();
+				range.rangeType = CalComponentRangeType.Single;
+				range.dt = cdt;
+
+				IntPtr raw = Marshal.AllocHGlobal (Marshal.SizeOf (range));
+				Marshal.StructureToPtr (range, raw, false);
+
+				e_cal_component_set_recurid (Handle, raw);
+				Marshal.FreeHGlobal (raw);
+			}
+		}
+#endif
+
 		[DllImport("ecal")]
 			static extern void e_cal_component_get_summary (IntPtr raw, ref CalComponentText text);
 		[DllImport("ecal")]
@@ -312,6 +388,26 @@ namespace Evolution {
 			}
 		}
 
+#if USE_EDS_2_24
+		[DllImport("ecal")]
+			static extern void e_cal_component_get_transparency (IntPtr raw, ref CalTransparency trans);
+		[DllImport("ecal")]
+			static extern void e_cal_component_set_transparency (IntPtr raw, CalTransparency trans);
+
+		public CalTransparency Transparency {
+			get
+			{
+				CalTransparency tr = CalTransparency.None;
+				e_cal_component_get_transparency (Handle, ref tr);
+				return tr;
+			}
+			set 
+			{
+				e_cal_component_set_transparency (Handle, value);
+			}
+		}
+#endif
+
 		[DllImport("ecal")]
 			static extern void e_cal_component_get_location (IntPtr raw, out IntPtr location);
 		[DllImport("ecal")]
@@ -330,6 +426,21 @@ namespace Evolution {
 			}
 		}
 
+#if USE_EDS_2_24
+		[DllImport("ecal")]
+			static extern bool e_cal_component_is_allday (IntPtr raw);
+
+		public bool IsAllDay
+		{
+			get
+			{
+				bool ret = e_cal_component_is_allday (Handle);
+
+				return ret;
+			}
+		}
+#endif
+
 		[DllImport("ecal")]
 			static extern void e_cal_component_get_dtstart (IntPtr raw, IntPtr dt);
 		[DllImport("ecal")]
@@ -370,6 +481,24 @@ namespace Evolution {
 				Marshal.FreeHGlobal (native_dt);
 			}
 		}
+
+		public CalComponentDateTime DtEnd {
+			get {
+				CalComponentDateTime dt = new CalComponentDateTime ();
+				IntPtr native_dt = GLib.Marshaller.StructureToPtrAlloc (dt);
+				e_cal_component_get_dtend(Handle, native_dt);
+				dt = (CalComponentDateTime) Marshal.PtrToStructure (native_dt, typeof (CalComponentDateTime));
+				Marshal.FreeHGlobal (native_dt);
+				
+				return dt;
+			}
+			set {	
+				IntPtr native_dt = GLib.Marshaller.StructureToPtrAlloc (value);
+				e_cal_component_set_dtend(Handle, native_dt);
+				Marshal.FreeHGlobal (native_dt);
+			}
+		}
+
 #endif		
 
 		[DllImport("ecal")]
@@ -434,6 +563,8 @@ namespace Evolution {
 
 		[DllImport("ecal")]
 			static extern void e_cal_component_get_last_modified (IntPtr raw, out IntPtr dt);
+		[DllImport("ecal")]
+			static extern void e_cal_component_set_last_modified (IntPtr raw, IntPtr dt);
 
 		public DateTime LastModified {
 			get {
@@ -441,6 +572,12 @@ namespace Evolution {
 				e_cal_component_get_last_modified (Handle, out dt);
 				return CalUtil.icaltimetype_to_datetime (dt, true);
 			}
+			set
+			{
+				IntPtr dt = CalUtil.datetime_to_icaltimetype (value);
+				e_cal_component_set_last_modified (Handle, dt);
+				CalUtil.e_cal_glue_free_icaltimetype (dt);
+			}
 		}
 
 		[DllImport("ecal")]
@@ -506,8 +643,7 @@ namespace Evolution {
 							for (i = 0, iter=raw_ret; iter != IntPtr.Zero; 
 							     iter = GLibUtil.gtksharp_slist_get_next (iter), i++) {
 								IntPtr data = GLibUtil.gtksharp_slist_get_data (iter);
-								CalComponentAttendee a = (CalComponentAttendee) Marshal.PtrToStructure (data, 
-																	typeof (CalComponentAttendee));
+								CalComponentAttendee a = (CalComponentAttendee) Marshal.PtrToStructure (data, typeof (CalComponentAttendee));
 								att_ret [i] = a;
 							}
 						}
@@ -525,6 +661,25 @@ namespace Evolution {
 			}
 		}
 
+		[DllImport("ecal")]
+			static extern void e_cal_component_get_organizer (IntPtr raw, ref CalComponentOrganizer org);
+		[DllImport("ecal")]
+			static extern void e_cal_component_set_organizer (IntPtr raw, ref CalComponentOrganizer org);
+		
+		public CalComponentOrganizer Organizer {
+			get 
+			{
+				CalComponentOrganizer org = new CalComponentOrganizer ();
+				e_cal_component_get_organizer (Handle, ref org);
+				return org;
+			}
+			set
+			{
+				e_cal_component_set_organizer (Handle, ref value);
+			}
+		}
+
+
 		[DllImport("ecal")]
 			static extern void e_cal_component_get_description_list (IntPtr raw, out IntPtr list);
 		[DllImport("ecal")]
@@ -590,7 +745,8 @@ namespace Evolution {
 				e_cal_component_get_categories_list(Handle, out raw_ret);
 				return CalUtil.GSListToStringArray (raw_ret, true);
 			}
-			set {
+			set 	
+			{
 				IntPtr list = GLibUtil.StringArrayToGSList (value);
 				e_cal_component_set_categories_list (Handle, list);
 				GLibUtil.FreeGSList (list);
@@ -605,9 +761,31 @@ namespace Evolution {
 
 			return ical;
 		}
-
+		
+#if USE_EDS_2_24
 		[DllImport("ecal")]
-			static extern int e_cal_component_commit_sequence (IntPtr handle);
+			static extern int e_cal_component_get_sequence_as_int (IntPtr handle);
+		[DllImport("ecal")]
+			static extern void e_cal_component_set_sequence (IntPtr handle, ref int set);
+		public int Sequence {
+			get 
+			{
+				int ret;
+				int raw = e_cal_component_get_sequence_as_int (Handle);
+				ret = raw;
+
+				return ret;
+				
+			}
+			set
+			{
+				e_cal_component_set_sequence (Handle, ref value);
+			}
+		}
+#endif
+		
+		[DllImport("ecal")]
+			static extern void e_cal_component_commit_sequence (IntPtr handle);
 		[DllImport("ecal")]
 			static extern int e_cal_component_abort_sequence (IntPtr handle);
 		[DllImport("ecal")]
@@ -659,6 +837,163 @@ namespace Evolution {
 	    {
 		e_cal_glue_free_cal_component (handle);
 	    }
+		
+#if USE_EDS_2_24
+	    [DllImport("ecal")]
+			static extern void e_cal_component_get_rrule_list (IntPtr handle, out IntPtr list);
+	    [DllImport("ecal")]
+		    static extern void e_cal_component_set_rrule_list (IntPtr handle, IntPtr list);
+	
+	    public CalRecurrence [] RRules
+	    {
+		    get 
+		    {
+			    IntPtr raw_ret = IntPtr.Zero;
+			    IntPtr iter = IntPtr.Zero;
+			    int i = 0;
+			    int len = 0;
+			    CalRecurrence[] ret = null;
 
+			    e_cal_component_get_rrule_list (Handle, out raw_ret);
+
+			    if (raw_ret != IntPtr.Zero) {
+				    if ((len = GLibUtil.g_slist_length (raw_ret)) > 0 ) {
+					    ret = new CalRecurrence [len];
+
+					    for (i = 0, iter=raw_ret; iter != IntPtr.Zero; 
+							    iter = GLibUtil.gtksharp_slist_get_next (iter), i++) {
+						    IntPtr data = GLibUtil.gtksharp_slist_get_data (iter);
+						    CalRecurrence r = (CalRecurrence) Marshal.PtrToStructure (data, typeof (CalRecurrence));
+
+						    ret [i] = r;
+					    }
+				    }
+				    GLibUtil.g_slist_free (raw_ret);
+			    }
+			    return ret == null ? new CalRecurrence [0] : ret;
+		    }
+		    set 
+		    {
+			    IntPtr ptr = IntPtr.Zero;
+			    IntPtr raw = IntPtr.Zero;
+
+			    foreach (CalRecurrence r in value) {
+				    ptr = Marshal.AllocHGlobal (Marshal.SizeOf (r));
+				    Marshal.StructureToPtr (r, ptr, false);
+				    raw = GLibUtil.g_slist_append (raw, ptr);
+			    }
+
+			    e_cal_component_set_rrule_list (Handle, raw);
+			    GLibUtil.g_slist_free (raw);
+		    }
+	    }
+
+	    [DllImport("ecal")]
+		    static extern bool e_cal_component_has_rrules (IntPtr handle);
+	    [DllImport("ecal")]
+		    static extern bool e_cal_component_has_exdates (IntPtr handle);
+	    [DllImport("ecal")]
+		    static extern bool e_cal_component_is_instance (IntPtr handle);
+
+	    public bool HasRRules 
+	    {
+		get {
+			bool ret = e_cal_component_has_rrules (Handle);
+			return ret;
+		}
+	    } 
+
+	    public bool HasExDates 
+	    {
+		get {
+			bool ret = e_cal_component_has_exdates (Handle);
+			return ret;
+		}
+	    }
+
+	    public bool IsInstance
+	    {
+		get
+		{
+			bool ret = e_cal_component_is_instance (Handle);
+			return ret;
+		}
+	    }
+	    
+	    [DllImport("ecal")]
+		    static extern void e_cal_component_get_exdate_list (IntPtr handle, out IntPtr list);
+	    [DllImport("ecal")]
+		    static extern void e_cal_component_set_exdate_list (IntPtr handle, IntPtr list);
+
+	    public CalComponentDateTime[] ExDates
+	    {
+		get
+		{
+			IntPtr raw_ret = IntPtr.Zero;
+			IntPtr iter = IntPtr.Zero;
+			int i = 0;
+			int len = 0;
+			CalComponentDateTime [] ret = null;
+		
+			e_cal_component_get_exdate_list (Handle, out raw_ret);
+			if (raw_ret != IntPtr.Zero) {
+				    if ((len = GLibUtil.g_slist_length (raw_ret)) > 0 ) {
+					    ret = new CalComponentDateTime [len];
+
+					    for (i = 0, iter=raw_ret; iter != IntPtr.Zero; 
+							    iter = GLibUtil.gtksharp_slist_get_next (iter), i++) {
+						    IntPtr data = GLibUtil.gtksharp_slist_get_data (iter);
+						    CalComponentDateTime d = (CalComponentDateTime) Marshal.PtrToStructure (data, typeof (CalComponentDateTime));
+
+						    ret [i] = d;
+					    }
+				    }
+				    GLibUtil.g_slist_free (raw_ret);
+			    }
+			    return ret == null ? new CalComponentDateTime [0] : ret;
+
+		}
+		set
+		{
+			IntPtr ptr = IntPtr.Zero;
+			IntPtr raw = IntPtr.Zero;
+
+			foreach (CalComponentDateTime d in value) {
+				ptr = Marshal.AllocHGlobal (Marshal.SizeOf (d));
+				Marshal.StructureToPtr (d, ptr, false);
+				raw = GLibUtil.g_slist_append (raw, ptr);
+			}
+
+			e_cal_component_set_exdate_list (Handle, raw);
+			GLibUtil.g_slist_free (raw);
+		}
+	    }
+	    
+	    [DllImport("ecal")]
+		    static extern void e_cal_component_set_x_prop (IntPtr handle, string name, string val);
+	    [DllImport("ecal")]
+		    static extern string e_cal_component_get_x_prop (IntPtr handle, string name);
+	    [DllImport("ecal")]
+		    static extern int e_cal_component_get_x_prop_as_int (IntPtr handle, string name);
+
+	    public void SetXProp (string name, string val)
+	    {
+		e_cal_component_set_x_prop (Handle, name, val);
+	    }
+
+	    public string GetXProp (string name)
+	    {
+		string ret = e_cal_component_get_x_prop (Handle, name);
+
+		return ret;
+	    }
+
+	    public int GetXPropAsInt (string name)
+	    {
+		int ret = e_cal_component_get_x_prop_as_int (Handle, name);
+
+		return ret;
+	    }
 	}
+#endif
 }
